import java.io.*;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import com.mindfusion.diagramming.*;

public class AnchorsFrame extends JFrame
{
	public AnchorsFrame()
	{
        initFrame();
      	initDiagram();
    }

    private void initDiagram()
	{
    	diagram.setSnapToAnchor(SnapToAnchor.OnCreateOrModify);
    	diagram.setRouteLinks(true);

    	diagram.getNodeEffects().add(new GlassEffect());
		diagram.getNodeEffects().add(new AeroEffect());
		
		diagram.getShapeNodeStyle().setBrush(
			new GradientBrush(Color.lightGray, new Color(0, 0xDE, 0xD1), 30));
		diagram.getDiagramStyle().setShadowBrush(
			new SolidBrush(new Color(0xBE, 0xBE, 0xBE, 0x96)));
		
		ShapeNode pb1, pb2, pb3, pb4, decb1, decb2;

		apat1 = new AnchorPattern(new AnchorPoint[]
		{
			new AnchorPoint(50, 0, true, true),
			new AnchorPoint(100, 50, true, true),
			new AnchorPoint(50, 100, true, true),
			new AnchorPoint(0, 50, true, true)
		}, "rhombus1");

		apat2 = new AnchorPattern(new AnchorPoint[]
		{
			new AnchorPoint(0, 0, true, false, MarkStyle.Circle, Color.blue),
			new AnchorPoint(50, 0, true, false, MarkStyle.Circle, Color.blue),
			new AnchorPoint(100, 0, true, false, MarkStyle.Circle, Color.red),
			new AnchorPoint(00, 100, false, true, MarkStyle.Rectangle),
			new AnchorPoint(50, 100, false, true, MarkStyle.Rectangle),
			new AnchorPoint(100, 100, false, true, MarkStyle.Rectangle),
			new AnchorPoint(0, 50, true, true, MarkStyle.Custom)
		}, "rhombus2");

		decisionShape = com.mindfusion.diagramming.Shape.fromId("Decision");

		pb1 = diagram.getFactory().createShapeNode(10, 7, 25, 18);
		pb1.setShape(com.mindfusion.diagramming.Shape.fromId("Ellipse"));
		pb1.setText("Start");
		pb1.setAnchorPattern(apat1);

		pb2 = diagram.getFactory().createShapeNode(20, 75, 25, 18);
		pb2.setText("node 1");
		pb2.setAnchorPattern(apat2);

		pb3 = diagram.getFactory().createShapeNode(70, 70, 25, 18);
		pb3.setText("node 2");
		pb3.setAnchorPattern(apat2);

		pb4 = diagram.getFactory().createShapeNode(80, 100, 25, 18);
		pb4.setShape(com.mindfusion.diagramming.Shape.fromId("Ellipse"));
		pb4.setText("End");
		pb4.setAnchorPattern(apat1);
		
		decb1 = diagram.getFactory().createShapeNode(20, 35, 30, 20);
		decb1.getShape().getId().equals("Rhombus");
		decb1.setText("check 1");
		decb1.setShape(decisionShape);
		decb1.setAnchorPattern(AnchorPattern.fromId("Decision1In3Out"));

		decb2 = diagram.getFactory().createShapeNode(70, 30, 30, 20);
		decb2.setShape(com.mindfusion.diagramming.Shape.fromId("rhombusDef"));
		decb2.setText("check 2");
		decb2.setShape(decisionShape);
		decb2.setAnchorPattern(AnchorPattern.fromId("Decision2In2Out"));

		diagram.getFactory().createDiagramLink(decb1, decb2);
		
		diagram.addDiagramListener(new DiagramAdapter()
		{
			public void nodeCreated(NodeEvent e)
			{
				ShapeNode node = ((ShapeNode)e.getNode());
				if(diagram.getDefaultShape() == com.mindfusion.diagramming.Shape.fromId("Rectangle"))
					node.setAnchorPattern(apat2);
				else
					node.setAnchorPattern(AnchorPattern.fromId("Decision1In3Out"));
			}
		});
	}
    
    private void initFrame()
    {
    	setTitle("MindFusion.Diagramming Sample: Anchor Points");
        setSize(new Dimension(784, 580));
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setLocationByPlatform(true);

        diagram = new Diagram();
    	diagView = new DiagramView(diagram);
    	txtArea = new JTextArea();
        label2 = new JLabel();
        rButNode = new  JRadioButton();
        rButDec = new  JRadioButton();
        butLoad = new JButton();
        butSave = new JButton();
        butPrinPreview = new JButton();
        scrPane = new JScrollPane(diagView);
    	grpButton = new ButtonGroup();
        _fileDialog = new JFileChooser();

        //
        //diag
        //
		diagram.setBackBrush(new SolidBrush(new Color((int)0xE0, (int)0xE0, (int)0xE0)));
		diagram.setLinkShape(LinkShape.Cascading);
		diagram.setFont(new Font("Arial", Font.PLAIN, 4));
		diagram.setLinkSegments((short)3);

        //
        //txtArea1
        //
        txtArea.setBackground(Color.white);
        txtArea.setName("label1");
        txtArea.setText("This sample shows how to define custom anchor-point patterns and assign them to boxes. There are also several predefines anchor patterns to choose from. One of them, DecisionBox, is used with the decision boxes in the sample diagram on the left. Link several boxes with arrows to get an idea how anchor points work.");
        txtArea.setEditable(false);
        txtArea.setAutoscrolls(false);
        txtArea.setLineWrap(true);
        txtArea.setWrapStyleWord(true);
        txtArea.setSize(155,150);
        //
		//label2
		//
		label2.setName("label1");
		label2.setSize(96, 16);
		label2.setText("Box shape");
		//
        //rButNode
        //
        rButNode.setName("rButNode");
        rButNode.setText("Node");
        rButNode.setSelected(true);
        rButNode.addActionListener(new ActionListener(){
        	public void actionPerformed(ActionEvent e){
				diagram.setDefaultShape(com.mindfusion.diagramming.Shape.fromId("Rectangle"));
			}
        });
        //
		// rButDec
		//
        rButDec.setName("rButDec");
		rButDec.setText("Decision");
		rButDec.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				diagram.setDefaultShape(decisionShape);
			}
		});
		//
		//grpButton
		//
		grpButton.add(rButNode);
		grpButton.add(rButDec);
		//
		//butLoad
		//
		butLoad.setName("butLoad");
		butLoad.setText("Load");
		butLoad.setPreferredSize(new Dimension(115,25));
		butLoad.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){

				if( _fileDialog.showOpenDialog(AnchorsFrame.this) == JFileChooser.APPROVE_OPTION)
					try{
						diagram.loadFrom(_fileDialog.getSelectedFile().getAbsolutePath());
					}
					catch(Exception exp){
						exp.getMessage();
					}
			}
		});
		//
		//butSave
		//
		butSave.setName("butSave");
		butSave.setText("Save");
		butSave.setPreferredSize(new Dimension(115,25));
		butSave.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){

				if(_fileDialog.showSaveDialog(AnchorsFrame.this) == JFileChooser.APPROVE_OPTION){

					try{
						diagram.saveTo(_fileDialog.getSelectedFile().getAbsolutePath());
					}
					catch (FileNotFoundException exp)
					{
						exp.getMessage();
					}
					catch (IOException exp)
					{
						exp.getMessage();
					}
				}
			}
		});

		//
		//butPrinPreview
		//
		butPrinPreview.setName("butPrinPreview");
		butPrinPreview.setText("Print Preview");
		butPrinPreview.setPreferredSize(new Dimension(115,25));
		butPrinPreview.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				diagView.getPrintOptions().setEnableEffects(true);
				diagView.printPreview();
			}
		});

		//
		//scrPane
		//
		scrPane.setName("spFcxScroll");
        scrPane.setSize(new Dimension());
        scrPane.setAutoscrolls(true);

        //fcxPanel
        JPanel fcxPanel = new JPanel();
        fcxPanel.setLayout(new BoxLayout(fcxPanel, BoxLayout.Y_AXIS));
       	fcxPanel.add(scrPane);

		//txtPanel
		JPanel txtPanel = new JPanel();
		txtPanel.setLayout(new BoxLayout(txtPanel, BoxLayout.Y_AXIS));
		txtPanel.add(txtArea);
	 	txtPanel.setSize(200,200);

		//compPanel
		JPanel compPanel = new JPanel();
		compPanel.setLayout(new java.awt.GridLayout(3,1));
		compPanel.setPreferredSize(new Dimension(130,80));
		compPanel.add(label2);
		compPanel.add(rButNode);
		compPanel.add(rButDec);

		//create Container
        Container cp = this.getContentPane();

		cp.add(fcxPanel);
		cp.add(txtPanel);
		cp.add(compPanel);
		cp.add(butLoad);
		cp.add(butSave);
		cp.add(butPrinPreview);

		//create SprLayout
		javax.swing.SpringLayout sprLayout = new javax.swing.SpringLayout();
		sprLayout.putConstraint(javax.swing.SpringLayout.EAST, txtPanel, -5,
        									 javax.swing.SpringLayout.EAST, cp );
		sprLayout.putConstraint(javax.swing.SpringLayout.NORTH, txtPanel, 5,
        									 javax.swing.SpringLayout.NORTH, cp );
        sprLayout.putConstraint(javax.swing.SpringLayout.EAST, compPanel, -5,
        									 javax.swing.SpringLayout.EAST, cp );
		sprLayout.putConstraint(javax.swing.SpringLayout.NORTH, compPanel, 245,
        									 javax.swing.SpringLayout.NORTH, cp );
        sprLayout.putConstraint(javax.swing.SpringLayout.EAST, cp, 165,
        									 javax.swing.SpringLayout.EAST, fcxPanel);
        sprLayout.putConstraint(javax.swing.SpringLayout.WEST, cp, 0,
        									javax.swing.SpringLayout.WEST, fcxPanel);
		sprLayout.putConstraint(javax.swing.SpringLayout.NORTH, cp , 0,
        									 javax.swing.SpringLayout.NORTH,fcxPanel);
        sprLayout.putConstraint(javax.swing.SpringLayout.SOUTH, cp , 0,
        									 javax.swing.SpringLayout.SOUTH,fcxPanel);
        sprLayout.putConstraint(javax.swing.SpringLayout.NORTH, butLoad , 400,
        									 javax.swing.SpringLayout.NORTH,cp);
        sprLayout.putConstraint(javax.swing.SpringLayout.EAST, butLoad , -20,
        									 javax.swing.SpringLayout.EAST,cp);
        sprLayout.putConstraint(javax.swing.SpringLayout.NORTH, butSave , 440,
        									 javax.swing.SpringLayout.NORTH,cp);
        sprLayout.putConstraint(javax.swing.SpringLayout.EAST, butSave , -20,
        									 javax.swing.SpringLayout.EAST,cp);
		sprLayout.putConstraint(javax.swing.SpringLayout.NORTH, butPrinPreview, 360,
        									 javax.swing.SpringLayout.NORTH,cp);
        sprLayout.putConstraint(javax.swing.SpringLayout.EAST, butPrinPreview , -20,
        									 javax.swing.SpringLayout.EAST,cp);

      	cp.setLayout(sprLayout);

        // Add window listener.
        this.addWindowListener
        (
            new WindowAdapter()
            {
                public void windowClosing(WindowEvent e)
                {
                	System.exit(0);
                }
            }
        );
    }

	Diagram diagram;
	DiagramView diagView;
	AnchorPattern apat1, apat2;
	com.mindfusion.diagramming.Shape decisionShape;
	
    JTextArea txtArea;
    JLabel label2;
    JRadioButton rButNode;
    JRadioButton rButDec;
    JButton butLoad;
    JButton butSave;
    JButton butPrinPreview;
    JScrollPane scrPane;
    ButtonGroup grpButton;
    JFileChooser _fileDialog;
	JFrame dialogFrame;
	
	private static final long serialVersionUID = 1L;
}