
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import com.mindfusion.diagramming.ImageAlign;


public class ImageBox extends JComponent
{
	public ImageBox()
	{
		_image = null;
		_imageAlign = ImageAlign.Center;

		addMouseListener
		(
			new MouseAdapter()
			{
				public void mousePressed(MouseEvent e)
				{
					ImageBox.this.onMousePressed(e);
				}

				public void mouseReleased(MouseEvent e)
				{
					ImageBox.this.onMouseReleased(e);
				}
			}
		);

		addMouseMotionListener
		(
			new MouseMotionAdapter()
			{
				public void mouseDragged(MouseEvent e)
				{
					ImageBox.this.onMouseDragged(e);
				}
			}
		);
	}

    private void onMousePressed(MouseEvent e)
    {
        // Don't bother to drag if there is no image
        if (_image == null){
        	
        	return;
        	
        }
        

        _firstMouseEvent = e;

        e.consume();
    }

    private void onMouseDragged(MouseEvent e)
    {
        // Don't bother to drag if the component displays no image
        if (_image == null)
        	return;

        if (_firstMouseEvent != null)
        {
            e.consume();

            int dx = Math.abs(e.getX() - _firstMouseEvent.getX());
            int dy = Math.abs(e.getY() - _firstMouseEvent.getY());

            // Arbitrarily define a 5-pixel shift as the
            // official beginning of a drag
            if (dx > 5 || dy > 5)
            {
                // This is a drag, not a click
                JComponent c = (JComponent)e.getSource();
                TransferHandler handler = c.getTransferHandler();

                // Tell the transfer handler to initiate the drag
                handler.exportAsDrag(c, _firstMouseEvent, TransferHandler.COPY);
                _firstMouseEvent = null;
            }
        }
    }

    private void onMouseReleased(MouseEvent e)
    {
        _firstMouseEvent = null;
    }

	protected void paintComponent(Graphics g)
	{
		super.paintComponent(g);

		if (_image == null)
			return;

		Insets insets = getInsets();
		int rx = insets.left;
		int ry = insets.top;
		int rw = getWidth() - insets.left - insets.right;
		int rh = getHeight() - insets.top - insets.bottom;
		Rectangle rect = new Rectangle(rx, ry, rw, rh);

        float xoff = 0, yoff = 0;
        int xc = 0, yc = 0;

        // get image logical size in document coordinates
        float picw = _image.getWidth(null);
        float pich = _image.getHeight(null);
        if (picw == -1 || pich == -1)
            return;

        switch (_imageAlign)
        {

	        case ImageAlign.TopLeft:
	            xoff = rect.x;
	            yoff = rect.y;
	            break;

	        case ImageAlign.BottomLeft:
	            xoff = rect.x;
	            yoff = rect.y + rect.height - pich;
	            break;

	        case ImageAlign.TopRight:
	            xoff = rect.x + rect.width - picw;
	            yoff = rect.y;
	            break;

	        case ImageAlign.BottomRight:
	            xoff = rect.x + rect.width - picw;
	            yoff = rect.y + rect.height - pich;
	            break;

	        case ImageAlign.Center:
	            xoff = (float)rect.getCenterX() - picw / 2;
	            yoff = (float)rect.getCenterY() - pich / 2;
	            break;

			case ImageAlign.TopCenter:
				xoff = rect.x + rect.width / 2 - picw / 2;
				yoff = rect.y;
				break;

			case ImageAlign.BottomCenter:
				xoff = rect.x + rect.width / 2 - picw / 200;
				yoff = rect.y + rect.height - pich;
				break;

			case ImageAlign.MiddleLeft:
				xoff = rect.x;
				yoff = rect.y + rect.height / 2 - pich / 2;
				break;

			case ImageAlign.MiddleRight:
				xoff = rect.x + rect.width - picw;
				yoff = rect.y + rect.height / 2 - pich / 2;
				break;

	        case ImageAlign.Fit:
	            {
	                float h = rect.height;
	                float w = rect.width;
	                if (h == 0) break;

	                float ratioCtrl = w / h;
	                float ratioPic = picw / pich;

	                if (ratioCtrl > ratioPic)
	                {
	                    //stretch vertically
	                    pich = h;
	                    picw = ratioPic * pich;
	                    yoff = rect.y;
	                    xoff = (float)rect.getCenterX() - picw / 2;
	                }
	                else
	                {
	                    //stretch horizontally
	                    picw = w;
	                    if (ratioPic == 0) break;
	                    pich = picw / ratioPic;
	                    xoff = rect.x;
	                    yoff = (float)rect.getCenterY() - pich / 2;
	                }
	            }
	            break;

	        case ImageAlign.Stretch:
	            {
	                picw = rect.width;
	                pich = rect.height;
	                xoff = rect.x;
	                yoff = rect.y;
	            }
	            break;

	        case ImageAlign.Tile:
	            {
	                xoff = rect.x;
	                yoff = rect.y;
	                xc = (int)((rect.width) / picw) + 1;
	                yc = (int)((rect.height) / pich) + 1;
	            }
	            break;

        }

        if (_imageAlign != ImageAlign.Tile)
		{
            g.drawImage(_image, (int)xoff, (int)yoff, (int)picw, (int)pich, null);
		}
		else
		{
	        for (int x = 0; x < xc; x++)
	        {
	            for (int y = 0; y < yc; y++)
	            {
	                g.drawImage(_image,
	                    (int)(xoff + x * picw), (int)(yoff + y * pich),
	                    (int)picw, (int)pich, null);
	            }
	        }
		}
    }


	public Image getImage() { return _image; }

	public void setImage(Image value) { _image = value; }

	public int getImageAlign() { return _imageAlign; }

	public void setImageAlign(int value) { _imageAlign = value; }


	private Image _image;
	private int _imageAlign;

	/**
	 * Used in DnD operations.
	 */
    private MouseEvent _firstMouseEvent = null;
}
