import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.Rectangle;
import java.awt.geom.AffineTransform;
import java.awt.geom.Dimension2D;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;

import com.mindfusion.diagramming.Diagram;
import com.mindfusion.diagramming.DiagramNode;
import com.mindfusion.diagramming.ExternalizableImage;
import com.mindfusion.diagramming.ImageAlign;
import com.mindfusion.diagramming.RenderOptions;
import com.mindfusion.diagramming.Utilities;


public class IconNode extends DiagramNode
{
	// required for serialization
	public IconNode()
	{
	}

	public IconNode(Diagram diagram)
	{
		super(diagram);
		
		drawSign();
		icon = defaultIcon;
		label = "Label";

		setBounds(getBounds().x, getBounds().y,
				calculateSize().width, calculateSize().height);
	}
	
	@Override
	protected void drawLocal(Graphics2D g, RenderOptions options)
	{
		// Draw the icon center at the top
		Utilities.drawImage(g, icon, getLocalBounds(), ImageAlign.TopCenter, getParent());

		g.setPaint(Color.black);
		g.setFont(new Font("Arial", Font.BOLD, 10));
		
		// Draw the label at the bottom
		Rectangle2D labelBounds = MainFrame.fromLTRB(
			(getBounds().width - g.getFontMetrics().stringWidth(label)) / 2, 
			getBounds().height, 
			getBounds().getMaxX(), 
			10);
		
		g.drawString(label,
			(float)labelBounds.getX(), (float)labelBounds.getY() - 1);
	}

	protected void updateCreate(Point2D current)
	{
		super.updateCreate(current);
		setBounds((float)current.getX(), (float)current.getY(), 
				(float)calculateSize().getWidth(), (float)calculateSize().getHeight());
	}
	
	@Override
	protected void drawShadow(Graphics2D g, RenderOptions options)
	{
	}

	private Dimension calculateSize()
	{
		Rectangle imageSize = new Rectangle(0, 0, 40, 33);

		Dimension2D textSize = getParent().measureString(label, new Font("Arial", Font.BOLD, 10));
		
		return new Dimension(
				(int)Math.max(imageSize.width, textSize.getHeight()),
				(int)(imageSize.height + textSize.getHeight() / 5));
	}
	
	static Rectangle2D deviceToDoc(Graphics2D g, Rectangle r)
	{
	    return deviceToDoc(g.getTransform(), r);
	}

	static Rectangle2D deviceToDoc(AffineTransform at, Rectangle r)
	{
	    try
	    {
	        // get the corner points
	        Point2D src0 = new Point2D.Double(r.x, r.y);
	        Point2D src1 = new Point2D.Double(r.x + r.width, r.y + r.height);
	        Point2D dst0 = new Point2D.Double();
	        Point2D dst1 = new Point2D.Double();

	        // get the world coordinates
	        at.inverseTransform(src0, dst0);
	        at.inverseTransform(src1, dst1);

	        // return rectangle in world coordinates
	        return MainFrame.fromLTRB(dst0.getX(), dst0.getY(), dst1.getX(), dst1.getY());
	    }
	    catch (NoninvertibleTransformException ex)
	    {
	        return new Rectangle2D.Double(0, 0, 0, 0);
	    }
	}
	
	private void drawSign()
	{
		defaultIcon = new BufferedImage(32, 32, BufferedImage.TYPE_INT_ARGB);
		Graphics2D graphics = defaultIcon.createGraphics();
		
		Paint p = graphics.getPaint();
		Font f = graphics.getFont();
	
		graphics.setPaint(Color.black);
		graphics.setFont(new Font("Arial", Font.BOLD, 40));
		graphics.drawString("?", 4, 35);
		graphics.setFont(f);
		graphics.setPaint(p);

		graphics.dispose();
	}
	
	public Rectangle2D getEditRect(Point2D mousePosition)
	{
		return getBounds();
	}
	
	public String getTextToEdit()
	{
		return label;
	}

	public void setEditedText(String newText)
	{
		label = newText;
	}
	
	public BufferedImage getIcon()
	{
		return icon;
	}

	public void setIcon(BufferedImage icon)
	{
		this.icon = icon;
		this.repaint();
	}

	public String getLabel()
	{
		return label;
	}

	public void setLabel(String label)
	{
		this.label = label;
	}
	
	@Override
	public void writeExternal(ObjectOutput out) throws IOException
	{
		super.writeExternal(out);
		
		out.writeObject(ExternalizableImage.getExImage(icon));
		out.writeObject(label);
	}
	
	@Override
	public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException
	{
		super.readExternal(in);
		
		icon = (BufferedImage)((ExternalizableImage)in.readObject()).getImage();
		label = (String)in.readObject();
	}
	
	private BufferedImage icon;
	private String label;

	static private BufferedImage defaultIcon;
	static private final long serialVersionUID = 1L;
}