import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.Random;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;

import com.mindfusion.diagramming.ArrowHeads;
import com.mindfusion.diagramming.Behavior;
import com.mindfusion.diagramming.Diagram;
import com.mindfusion.diagramming.DiagramAdapter;
import com.mindfusion.diagramming.DiagramEvent;
import com.mindfusion.diagramming.DiagramView;
import com.mindfusion.diagramming.FitSize;
import com.mindfusion.diagramming.GraphicsUnit;
import com.mindfusion.diagramming.ImageAlign;
import com.mindfusion.diagramming.ModificationStart;
import com.mindfusion.diagramming.NodeEvent;
import com.mindfusion.diagramming.RenderTarget;
import com.mindfusion.diagramming.ShapeNode;
import com.mindfusion.diagramming.SolidBrush;

public class MainFrame extends JFrame
{
	

	public MainFrame()
	{
		initDiagram();
		initFrame();
	}

	private void initDiagram()
	{
		diagram = new Diagram();
		diagram.setLinkHeadShape(ArrowHeads.Arrow);
		diagram.setLinkHeadShapeSize(10);
		diagram.setAdjustmentHandlesSize(4);
		diagram.setBackBrush(new SolidBrush(Color.white));
		diagram.setBackgroundImageAlign(ImageAlign.TopLeft);
		diagram.setDynamicLinks(true);
		diagram.setMeasureUnit(GraphicsUnit.Point);

		diagram.addDiagramListener(new DiagramAdapter()
		{
			public void nodeDoubleClicked(NodeEvent e)
			{	
				onNodeDoubleClicked(e);
			}
			
			public void clicked(DiagramEvent e)
			{	
				onClicked(e);
			}
		});
		
		diagramView = new DiagramView(diagram);
		diagramView.setBehavior(Behavior.LinkShapes);
		diagramView.setDiagram(diagram);
		diagramView.setInplaceEditFont(new Font("Microsoft Sans Serif", Font.PLAIN, 8));
		diagramView.setModificationStart(ModificationStart.AutoHandles);
		diagramView.getPrintOptions().setRenderTarget(RenderTarget.Printer);

		// Enable serialization of IconNode instances
		Diagram.registerItemClass(IconNode.class.getClass(), "IconNode", 1);
		
		// Let users draw IconNode objects
		diagramView.setCustomNodeType(IconNode.class);
		diagramView.setBehavior(Behavior.Custom);
		
		// Show some info about this sample
		ShapeNode info = new ShapeNode(diagram);
		info.setBounds(5, 5, 400, 120);
		info.setText("This example shows how to create a custom node class that " +
			"exposes some new properties in addition to the ones defined by " +
			"DiagramNode and implements its own drawing logic. Click on the " +
			"drawing surface or draw with the mouse to create isntances of " +
			"the custom IconeNode class.");
		info.setBrush(new SolidBrush(Color.white));
		info.setLocked(true);
		info.setFont(new Font("Arial", Font.PLAIN, 10));
		info.resizeToFitText(FitSize.KeepRatio);
		info.setTransparent(true);
		diagram.getNodes().add(info);
		
		imageList = new Image[3];
		imageList[0] = getImage("Files\\ei0003-32.png");
		imageList[1] = getImage("Files\\ei0018-32.png");
		imageList[2] = getImage("Files\\ei0081-32.png");
	}
	
	Image[] imageList;
	
	public void onLoad()
	{
		JFileChooser fch = new JFileChooser();
	
		try
		{
			if (fch.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
				diagram.loadFrom(fch.getSelectedFile().getAbsolutePath());
		}
		catch (Exception exp)
		{
			JOptionPane.showMessageDialog(this, "Not a JDiagram file!");
		}
	}
	
	public void onSave()
	{
		JFileChooser fch = new JFileChooser();
		
		try
		{
			if (fch.showSaveDialog(this) == JFileChooser.APPROVE_OPTION)
				diagram.saveTo(fch.getSelectedFile().getAbsolutePath());
		}
		catch (Exception exp)
		{
			exp.printStackTrace();
			JOptionPane.showMessageDialog(this, "Error saving!");
		}
	}
	
	Random rnd = new Random();
	
	public void onNodeDoubleClicked(NodeEvent e)
	{	
		int i = rnd.nextInt() % 3;
		
		i = Math.abs(i);
		
		IconNode iconNode = (IconNode)e.getNode();
		if (iconNode != null)
			iconNode.setIcon((BufferedImage)imageList[i]);
	}
	
	public void onClicked(DiagramEvent e)
	{	
		IconNode iconNode = new IconNode(diagram);
		diagram.getNodes().add(iconNode);

		iconNode.moveTo(e.getMousePosition().x, e.getMousePosition().y);
	}
	
	private Image getImage(String fileName)
	{
		try
		{
			return javax.imageio.ImageIO.read(new java.io.File(fileName));
		}
		catch (IOException ex)
		{
			try
			{
				return javax.imageio.ImageIO.read(new java.io.File("..\\" + fileName));
			}
			catch (IOException ex2)
			{
				return null;
			}
		}
	}
	
	public static Rectangle2D fromLTRB(double l, double t, double r, double b)
	{
		return new Rectangle2D.Double(l, t, r - l, b - t);
	}
	
	private void initFrame()
	{
		JMenuItem mILoad = new JMenuItem();
		mILoad.setName("mILoad");
		mILoad.setText("Load");
		mILoad.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent arg0)
			{
				onLoad();
			}
		});
		
		JMenuItem mISave = new JMenuItem();
		mISave.setName("mISave");
		mISave.setText("Save");
		mISave.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent arg0)
			{
				onSave();
			}
		});
		
		JMenu mFile = new JMenu();
		mFile.setName("mFile");
		mFile.setText("File");
		mFile.add(mILoad);
		mFile.add(mISave);
		
		JMenuBar mBar = new JMenuBar();
		mBar.setName("mBar");
		mBar.add(mFile);
		
		Dimension size = Toolkit.getDefaultToolkit().getScreenSize();
		setSize(size);
		setTitle("MindFusion.Diagramming Sample: Icon Nodes");
		setJMenuBar(mBar);
		
		addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				System.exit(0);
			}
		});

		JScrollPane scrPane = new JScrollPane(diagramView); 
		setLayout(new GridLayout(1,1));
		add(scrPane);
	}
	
	private Diagram diagram;
	private DiagramView diagramView;
	
	private static final long serialVersionUID = 1L;
}